import json

from django.conf import settings
from django.template import Context, Template
from django.http import HttpResponse

from .helpers import get_translations


js_catalog_template = r"""
{% autoescape off %}
(function (globals) {

  var django = globals.django || (globals.django = {});

  /* gettext library */

  django.translations = {{ catalog_str }};

  django.localize = function (key, fallback, placeholders) {
    fallback = typeof(fallback) == "undefined" ? key : fallback;
    var value = django.translations[key];
    if (typeof(value) == 'undefined') {
      {% if not debug %}
      value = fallback;
      {% else %}
      value = '\{\{' + key + '\}\}';
      {% endif %}
    }
    if (placeholders && placeholders['plural']) {
      var plural = parseInt(placeholders['plural']);
      var values = value.split('|');
      delete placeholders['plural'];
      if (plural == 1 || values.length < 2) {
        value = value.split('|')[0];
      } else {
        value = value.split('|')[1];
      }
    }
    for (var ph in placeholders) {
      value = value.replace(':' + ph, placeholders[ph]);
    }
    return value;
  };

  /* formatting library */

  globals.localize = django.localize;

}(this));
{% endautoescape %}
"""


def render_javascript_catalog(translations=None):
    def _indent(s):
        return s.replace('\n', '\n  ')

    template = Template(js_catalog_template)
    context = Context({
        'catalog_str': _indent(json.dumps(
            translations, sort_keys=True, indent=2)) if translations else {},
        'debug': settings.LOCALIZE_DEBUG
    })

    return HttpResponse(template.render(context), 'text/javascript')


def javascript_catalog(request, domain='globaljs', packages=None):
    locale = request.LANGUAGE_CODE

    translations = get_translations(locale, domain)
    return render_javascript_catalog(translations)



import os
import toml

from collections import defaultdict

from django.conf import settings


_localize_translations = defaultdict(dict)
_localize_filemtimes = defaultdict(dict)
_localize_file_paths = defaultdict(dict)


def parse_translations_file(lang_path):
    file_path = os.path.abspath(lang_path)

    if os.path.exists(file_path):
        with open(file_path, 'r') as f:
            translations = toml.loads(f.read())
    else:
        translations = {}
    return translations


def get_translations(lang_code, domain='global', fallback=True):
    global _localize_translations
    global _localize_filemtimes
    global _localize_file_paths

    if domain not in _localize_file_paths[lang_code]:
        filename = '{}.toml'.format(domain)
        lang_path = os.path.join(settings.LOCALE_PATHS[0], lang_code, filename)
        # Fallback to default file (ie. for English)
        if fallback and not os.path.exists(lang_path):
            lang_path = os.path.join(
                settings.LOCALE_PATHS[0], 'default', filename)
        _localize_file_paths[lang_code][domain] = lang_path
    else:
        lang_path = _localize_file_paths[lang_code][domain]

    lang_translations = _localize_translations[lang_code]
    lang_filemtime = _localize_filemtimes[lang_code]

    translations = lang_translations.get(domain)

    if not translations:
        translations = parse_translations_file(lang_path)
        lang_translations[domain] = translations
        try:
            lang_filemtime[domain] = os.path.getmtime(
                os.path.abspath(lang_path))
        except OSError:
            pass
    else:
        _filemtime = lang_filemtime.get(domain)
        if os.path.getmtime(os.path.abspath(lang_path)) > _filemtime:
            translations = parse_translations_file(lang_path)
            lang_translations[domain] = translations
            try:
                lang_filemtime[domain] = os.path.getmtime(
                    os.path.abspath(lang_path))
            except OSError:
                pass

    return translations


def save_translations(lang_code, translations, domain='global'):
    filename = '{}.toml'.format(domain)
    file_path = os.path.abspath(
        os.path.join(settings.LOCALE_PATHS[0], lang_code, filename))
    with open(file_path, 'w') as f:
        toml.dump(translations, f)


def translate_by_key(lang_code, key, default=None, **kwargs):
    translations = get_translations(lang_code)

    # Just return the key if no default is specified
    if settings.LOCALIZE_DEBUG:
        default = "{{%s}}" % key
    sent = translations.get(key.lower(), default or key)
    plural = kwargs.get('plural')
    if plural is not None:
        del kwargs['plural']
        sents = sent.split('|')
        if plural == 1:
            sent = sents[0]
        else:
            sent = sents[1]

    for k in kwargs:
        sent = sent.replace(':%s' % k, kwargs[k])

    if key.isupper():
        sent = sent.upper()
    elif key.istitle():
        sent = sent.title()
    elif key[:1].isupper():
        sent = sent[:1].upper() + sent[1:]
    return sent



import re

from django import template
from django.utils.translation import get_language

from ..helpers import translate_by_key


register = template.Library()


@register.simple_tag(takes_context=True)
def localize(context, key, default=None, **kwargs):
    tmp_kwargs = {}
    for k, v in kwargs.iteritems():
        tmp_v = v
        matches = re.findall(r'({{[ ]*([\w\.]+)[ ]*}})', tmp_v)
        for m in matches:
            if '.' in m[1]:
                _parts = m[1].split('.')
                value = context.get(_parts.pop(0))
                if value:
                    for part in _parts:
                        if any((type(value) is dict, type(value) is list)):
                            value = value[part]
                        else:
                            value = getattr(value, part)
            else:
                value = context.get(m[1])
            if value:
                tmp_v = re.sub(m[0], value, tmp_v)
        tmp_kwargs[k] = tmp_v
    lang_code = get_language()
    return translate_by_key(lang_code, key, default, **tmp_kwargs)



from django.utils.translation import get_language
from django.utils.functional import lazy
from django.utils import six

from .helpers import translate_by_key


def localize(key, default=None, **kwargs):
    lang_code = get_language()
    return translate_by_key(lang_code, key, default, **kwargs)


localize_lazy = lazy(localize, six.text_type)



import os
import sys
import toml
import re

from optparse import make_option
from unidecode import unidecode

from django.core.management import BaseCommand


PATTERNS = {
    'html': {
        'matcher': r'{% ?(trans[ ]+[\'"]([^{}<>]*?)[\'"]) ?%}',
        'sub': r'^trans[ ]+[\'"](.*?)[\'"]$',
        'replace': 'localize "{}" "{}"',
        'process_val': lambda val: val.replace('"', '\\"')
    },
    'js': {
        'matcher': r'[{ ,(](gettext\([\'"]([^{}<>]*?)[\'"]\))',
        'sub': r'gettext\([\'"](.*?)[\'"]\)',
        'replace': 'localize(\'{}\', \'{}\')',
        'process_val': lambda val: val.replace("'", "\\'")
    },
}


class Command(BaseCommand):
    translations = {}
    inv_translations = {}

    option_list = BaseCommand.option_list + (
        make_option('-t', '--transfile', dest='transfile',
                    help="Path to translation file"),
        make_option('-f', '--file', dest='file',
                    help='Path to the file to replace', default=None),
        make_option('-d', '--directory', dest='directory',
                    help='Directory to search in', default=None),
        make_option('-e', '--extension', dest='extension', default='js',
                    help='File Extension (eg. js, py)'),
        make_option('-m', '--maxlen', dest='maxlen', default=70,
                    help='Max length of characters for the string to replace'),
    )

    def search_and_replace(self, path, maxlen, verbose=False):
        global PATTERNS
        ext = os.path.splitext(path)[1]
        if ext == '.js':
            has_translation = self._search_and_replace(
                path, maxlen, verbose, patterns=PATTERNS['js'])
        if ext == '.html':
            has_translation = self._search_and_replace(
                path, maxlen, verbose, patterns=PATTERNS['html'])
            if has_translation:
                self._load_localize_template_tag(path)

    def _load_localize_template_tag(self, path):
        re_tt_extends = re.compile(r'{% extends [\'"](.*)[\'"] %}')
        add_tag = False
        with open(path, 'r') as f:
            contents = f.read()
            templatetags = re.findall(r'{% load (\w+) %}', contents)
            if 'localize' not in templatetags:
                with open('{}.new'.format(path), 'w') as fnew:
                    has_extends = False
                    if re_tt_extends.search(contents):
                        has_extends = True

                    lines = contents.splitlines()
                    linecount = len(lines)
                    new_lines = []

                    for ind in xrange(linecount):
                        line = lines[ind]
                        if not add_tag:
                            if has_extends:
                                if (line != '' and
                                        not re_tt_extends.search(line)):
                                    new_lines.append('{% load localize %}')
                                    add_tag = True
                            else:
                                new_lines.append('{% load localize %}')
                                add_tag = True
                        new_lines.append(line)
                    new_contents = '\n'.join(new_lines)
                    fnew.write(new_contents)
        if add_tag:
            os.remove(path)
            os.rename('{}.new'.format(path), path)

    def _search_and_replace(self, path, maxlen, verbose, patterns):
        has_translation = False
        with open(path, 'r') as f, open('{}.new'.format(path), 'w') as fnew:
            m = re.compile(patterns['matcher'])
            for line in f.readlines():
                matches = m.findall(line)
                if len(matches) > 0:
                    has_translation = True
                    replaced_line = line
                    for match in matches:
                        actual = match[0]
                        val = match[1]
                        if len(val) > maxlen:
                            continue

                        if val in self.inv_translations:
                            key = self.inv_translations[val]
                        else:
                            key = re.sub('[^A-Za-z0-9 +/&%]+', '', val)
                            if isinstance(key, unicode):
                                key = unidecode(key)
                            key = re.sub('_+', '_', key.lower().replace(
                                ' ', '_').replace('/', '_')
                                         .replace('&', '_')
                                         .replace('%', '_percent_')
                                         .replace('+', '_plus_')).strip('_')
                            if key not in self.translations:
                                self.translations[key] = val

                        replaced = re.sub(
                            patterns['sub'],
                            patterns['replace'].format(
                                key, patterns['process_val'](val)), actual)
                        replaced_line = replaced_line.replace(actual, replaced)
                    if verbose and line != replaced_line:
                        sys.stdout.write(line.replace('\n', '').strip() + '\n')
                        sys.stdout.write(replaced_line.strip() + '\n\n')
                    fnew.write(replaced_line)
                else:
                    fnew.write(line)
        if not has_translation:
            os.remove('{}.new'.format(path))
        else:
            os.remove(path)
            os.rename('{}.new'.format(path), path)
        return has_translation

    def handle(self, *args, **options):
        transfile_path = options['transfile']
        if not transfile_path:
            sys.exit('Option --transfile is required')

        maxlen = options['maxlen']
        verbose = True if options['verbosity'] == '1' else False

        if os.path.exists(transfile_path):
            with open(transfile_path, 'r') as tf:
                self.translations = toml.loads(tf.read())

        self.inv_translations = {
            v: k for k, v in self.translations.iteritems()}

        if options['file']:
            file_path = os.path.abspath(options['file'])
            if not os.path.isfile(file_path):
                sys.exit('{} is not a file or the file does not exist.'.format(
                    args.file))
            self.search_and_replace(file_path, maxlen, verbose)
        else:
            extension = '.{}'.format(options['extension'])
            for directory, _, filenames in os.walk(options['directory']):
                for filename in filenames:
                    if os.path.splitext(filename)[1] == extension:
                        file_path = os.path.join(directory, filename)
                        self.search_and_replace(file_path, maxlen, verbose)

        with open(transfile_path, 'w') as tf:
            toml.dump(self.translations, tf)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages

VERSION = (0, 0, 1)
__version__ = '.'.join(map(str, VERSION))

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

packages = find_packages()

setup(
    name='django-l10n',
    version=__version__,
    description='A different approach to Django localization',
    long_description=README,
    url='https://github.com/ralkan/django-l10n',
    download_url='https://github.com/ralkan/django-l10n/tarball/%s' % (
        __version__,),
    author='Resul Alkan',
    author_email='me@resulalkan.com',
    license='MIT',
    keywords='django,l10n,i18n,django language,django localization',
    packages=packages,
    install_requires=[
        'Django==1.7.10',
        'toml==0.9.4'
    ]
)
